/*
 * lilJudd
 * Copyright (C) 2024 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.liljudd.data.database.repository

import net.moonleay.liljudd.data.database.entry.TimePlanningChannelsData
import net.moonleay.liljudd.data.database.tables.TimePlanningChannels
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.and
import org.jetbrains.exposed.sql.deleteWhere
import org.jetbrains.exposed.sql.insert
import org.jetbrains.exposed.sql.selectAll
import org.jetbrains.exposed.sql.transactions.transaction

object TimePlanningChannelsRepository {

    fun getAll(): List<TimePlanningChannelsData> {
        val dataList = mutableListOf<TimePlanningChannelsData>()
        transaction {
            for (tp in TimePlanningChannels.selectAll())
                dataList.add(
                    TimePlanningChannelsData(
                        id = tp[TimePlanningChannels.id],
                        serverID = tp[TimePlanningChannels.serverid],
                        channelID = tp[TimePlanningChannels.channelid],
                    )
                )
        }
        return dataList
    }

    fun exists(channelID: Long, serverID: Long): Boolean =
        transaction {
            TimePlanningChannels.selectAll()
                .where { TimePlanningChannels.channelid eq channelID and (TimePlanningChannels.serverid eq serverID) }
                .firstOrNull() != null
        }

    fun delete(id: Int) {
        transaction {
            TimePlanningChannels.deleteWhere { TimePlanningChannels.id eq id }
        }
    }

    fun deleteFromChannelInServer(channelID: Long, serverID: Long) {
        transaction {
            TimePlanningChannels.deleteWhere { channelid eq channelID and (serverid eq serverID) }
        }
    }

    fun write(data: TimePlanningChannelsData): Int =
        transaction {
            TimePlanningChannels.insert {
                it[serverid] = data.serverID
                it[channelid] = data.channelID
            } get TimePlanningChannels.id
        }
}
